#===========================================================================================================
ccs_version = function() {

	return( "calencalcs version 1.1 2018-12-14" )
}

#===========================================================================================================
ccs_init_calendar = function( calname ) {

	if( storage.mode(calname) != 'character' )
		stop(paste("Error, supplied calendar name must be a character string. Examples: standard gregorian julian noleap 365_day 360_day proleptic_gregorian"))

	ccs_calendar_ptr = .Call( "R_ccs_init_calendar", calname )

	retval = list(
		name 	= calname,
		ptr	= ccs_calendar_ptr
		)

	class( retval ) = 'calencalcs_calendar'

	return( retval )
}

#===========================================================================================================
# Returns TRUE if the passed year is a leap year in the passed calendar, and FALSE otherwise.
#
ccs_isleap = function( calendar, year ) {

	if( storage.mode(calendar) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar"))

	if( class(calendar) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar is not a valid initialized calencalcs calendar"))

	if( (storage.mode(year) == 'list') && (!is.null(year$year)))
		year = as.integer(year$year)
	else if( (storage.mode(year) == 'double') || (storage.mode(year) == 'integer'))
		year = as.integer(year)
	else
		stop(paste("Error, passed year must be a utDate or a numeric. Storage mode passed:", 
			storage.mode(year) ))

	rv2i = .Call( "R_ccs_isleap", calendar$ptr, year )

	#---------------------------------------------
	# Returned value is a 2-element integer array; 
	# [1] is 0 on success, -1 on error
	# [2] is 1 if year is a leap year, 0 otherwise
	#---------------------------------------------
	if( rv2i[1] != 0 ) 
		stop(paste("Error in ccs_isleap when calling with calendar", calendar$name, "and year", year ))

	return( rv2i[2] == 1 )
}

#==============================================================================================================
# Given a month in a specified calendar, returns the number of days in that month
#
ccs_dpm = function( calendar, year, month ) {

	if( storage.mode(calendar) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar"))

	if( class(calendar) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar is not a valid initialized calencalcs calendar"))

	rv2i = .Call( "R_ccs_dpm", calendar$ptr, as.integer(year), as.integer(month) )

	#---------------------------------------------
	# Returned value is a 2-element integer array; 
	# [1] is 0 on success, -1 on error
	# [2] is returned dpm
	#---------------------------------------------
	if( rv2i[1] != 0 ) 
		stop(paste("Error in R_ccs_dpm when calling with calendar", calendar$name, "year", 
			year, 'month', month ))

	return( rv2i[2] )
}

#==============================================================================================================
# Given a date in a specified calendar, returns the (true) Julian day of the date. Note that
# true Julian days start at "1" in 1 Jan 4713 BC. January 1, 2000, was 2,451,545
#
ccs_date2jday = function( calendar, year, month, day ) {

	if( storage.mode(calendar) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar"))

	if( class(calendar) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar is not a valid initialized calencalcs calendar"))

	rv2i = .Call( "R_ccs_date2jday", calendar$ptr, as.integer(year), as.integer(month), as.integer(day) )

	#---------------------------------------------
	# Returned value is a 2-element integer array; 
	# [1] is 0 on success, -1 on error
	# [2] is returned Julian day number
	#---------------------------------------------
	if( rv2i[1] != 0 ) 
		stop(paste("Error in ccs_date2jday when calling with calendar", calendar$name, "and date", 
			year, '/', month, '/', day ))

	return( rv2i[2] )
}

#==============================================================================================================
# Given a Julian day number and a specified calendar, returns the date in that calendar as an
# array of 3 integers (year, month, day). Note that true Julian days start at "1" in 
# 1 Jan 4713 BC. January 1, 2000, was 2,451,545
#
ccs_jday2date = function( calendar, jday ) {

	if( storage.mode(calendar) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar"))

	if( class(calendar) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar is not a valid initialized calencalcs calendar"))

	rv4i = .Call( "R_ccs_jday2date", calendar$ptr, as.integer(jday) )

	#------------------------------------------------
	# Returned value is a 4-element integer array; 
	# [1] is 0 on success, -1 on error
	# [2] is returned year, [3] is month, [4] is day
	#------------------------------------------------
	if( rv4i[1] != 0 ) 
		stop(paste("Error in R_ccs_jday2date when calling with calendar", calendar$name, "and Julian day number ", 
			jday ))

	return( c(rv4i[2], rv4i[3], rv4i[4]) )
}

#==============================================================================================================
# Given a date in a specified calendar, returns the day-of-year number in that calendar (counting starts at 1)
#
ccs_date2doy = function( calendar, year, month, day ) {

	if( storage.mode(calendar) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar"))

	if( class(calendar) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar is not a valid initialized calencalcs calendar"))

	rv2i = .Call( "R_ccs_date2doy", calendar$ptr, as.integer(year), as.integer(month), as.integer(day) )

	#---------------------------------------------
	# Returned value is a 2-element integer array; 
	# [1] is 0 on success, -1 on error
	# [2] is returned day of year
	#---------------------------------------------
	if( rv2i[1] != 0 ) 
		stop(paste("Error in R_ccs_date2doy when calling with calendar", calendar$name, "and date", 
			year, '/', month, '/', day ))

	return( rv2i[2] )
}

#==============================================================================================================
# Given a day of year number (starting at 1) and a specified calendar, returns the date in that calendar as an
# array of 3 integers (year, month, day). 
#
ccs_doy2date = function( calendar, year, doy ) {

	if( storage.mode(calendar) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar"))

	if( class(calendar) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar is not a valid initialized calencalcs calendar"))

	rv4i = .Call( "R_ccs_doy2date", calendar$ptr, as.integer(year), as.integer(doy) )

	#------------------------------------------------
	# Returned value is a 4-element integer array; 
	# [1] is 0 on success, -1 on error
	# [2] is returned year, [3] is month, [4] is day
	#------------------------------------------------
	if( rv4i[1] != 0 ) 
		stop(paste("Error in R_ccs_doy2date when calling with calendar", calendar$name, "year", year, 
			"and day of year number ", doy ))

	return( c(rv4i[2], rv4i[3], rv4i[4]) )
}

#==============================================================================================================
# Given a date in one calendar, and the number of days since that date, this returns
# the date in a (possibly different) calendar. As a special case, specifying "zero" 
# days since and giving two different caleders essentially converts dates between calendars.
#
# Returns a 3-element integer array: c(year, month, day)
#
ccs_dayssince = function( calendar_orig, year_orig, month_orig, day_orig, ndays_since, calendar_new ) {

	if( storage.mode(calendar_orig) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar_orig"))

	if( class(calendar_orig) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar_orig is not a valid initialized calencalcs calendar"))

	if( storage.mode(calendar_new) != 'list')
		stop(paste("Error, passed a bad or uninitialized calendar_new"))

	if( class(calendar_new) != 'calencalcs_calendar')
		stop(paste("Error, passed calendar_new is not a valid initialized calencalcs calendar"))

	rv4i = .Call( "R_ccs_dayssince", calendar_orig$ptr, as.integer(year_orig), as.integer(month_orig),
		as.integer(day_orig), as.integer(ndays_since), calendar_new$ptr )

	#------------------------------------------------
	# Returned value is a 4-element integer array; 
	# [1] is 0 on success, -1 on error
	# [2] is returned year, [3] is month, [4] is day
	#------------------------------------------------
	if( rv4i[1] != 0 ) 
		stop(paste("Error in R_ccs_dayssince when calling with original calendar", calendar_orig$name, 
			"and original date ", year_orig, '/', month_orig, '/', day_orig ))

	return( c(rv4i[2], rv4i[3], rv4i[4]) )
}

#======================================================================================================================
utCalendar = function( value, unit_str, style='array', calendar='standard' ) {

	if( ! is.character(unit_str) ) {
		stop(paste("Error in call to calencalcs routine utCalendar: the second arg (specifying units) must be a character string!"))
		}

	if( ! is.double(value) )
		value <- as.double(value)

	if( style == 'list' )
		istyle <- 1
	else if( style == 'array' )
		istyle <- 2
	else
		stop(paste("Error, style arg must be either 'list' or 'array'.  Passed value:",style))

	rv <- .Call("R_utCalendar_ccs_v3p0",
		value,
		as.character(unit_str),
		as.integer(istyle),
		as.character(calendar),
		PACKAGE="calencalcs")

	return(rv)
}

#======================================================================================================================
# Convers a calendar date specified by a 'utDate' class object
# into a value (amount) given in the units specified by 'unit' (which is a 
# unit-spec string or 'udUnits' class object).  Returns that value.
# Added in version 1.2: date can be a list of objects of class 'utDate',
# in which case this returns a vector of values rather than a scalar.
#
utInvCalendar <- function( date, unit_str, calendar="standard" )
{
	if( ! is.character(unit_str) )
		stop(paste("Error in call to udunits routine utInvCalendar: the second arg (specifying units) must be a character string!"))

	STYLE_LIST_OF_DATES  = 1
	STYLE_LIST_OF_ARRAYS = 2
	STYLE_SINGLE         = 3

	if( class(date) == "utDate" ) {
		style = STYLE_SINGLE
		nn <- 1
		}
	else
		{
		if( class(date) == "list") {
			nn <- length(date)
			if( (nn >= 1) && (class(date[[1]]) == "utDate"))
				style = STYLE_LIST_OF_DATES
			else
				{
				has_year   = !is.null( date$year   )
				has_month  = !is.null( date$month  )
				has_day    = !is.null( date$day    )
				has_hour   = !is.null( date$hour   )
				has_minute = !is.null( date$minute )
				has_second = !is.null( date$second )
				
				if( has_year && has_month && has_day && has_hour && has_minute && has_second ) {
					style = STYLE_LIST_OF_ARRAYS
					nn = length( date$year )
					}
				else
					stop("utInvCalendar: I was passed a date that is neither of class 'utDate' nor a list of objects of class 'utDate' nor a list with arrays named year, month, day, etc.!")
				}
			}
		else
			stop(paste("Error in calencalcs version of utInvCalendar: passed date must be class 'utDate' or 'list', but is:", class(date) ))
		}

	rv <- list()
	rv$year   <- array(0,nn)
	rv$month  <- array(0,nn)
	rv$day    <- array(0,nn)
	rv$hour   <- array(0,nn)
	rv$minute <- array(0,nn)
	rv$secod  <- array(0.0,nn)
	rv$error  <- -1
	rv$value  <- double(nn)
	
	if( style == STYLE_SINGLE ) {
		rv$year   <- date$year
		rv$month  <- date$month
		rv$day    <- date$day
		rv$hour   <- date$hour
		rv$minute <- date$minute
		rv$second <- date$second
		}

	else if( style == STYLE_LIST_OF_DATES ) {
		for( i in 1:nn ) {
			rv$year[i]   <- date[[i]]$year
			rv$month[i]  <- date[[i]]$month
			rv$day[i]    <- date[[i]]$day
			rv$hour[i]   <- date[[i]]$hour
			rv$minute[i] <- date[[i]]$minute
			rv$second[i] <- date[[i]]$second
			}
		}

	else if( style == STYLE_LIST_OF_ARRAYS ) {
		rv$year   <- date$year
		rv$month  <- date$month
		rv$day    <- date$day
		rv$hour   <- date$hour
		rv$minute <- date$minute
		rv$second <- date$second
		}
	
	retval <- .Call("R_utInvCalendar_ccs",
		as.integer(nn),
		as.integer(rv$year),
		as.integer(rv$month),
		as.integer(rv$day),
		as.integer(rv$hour),
		as.integer(rv$minute),
		as.double (rv$second),
		as.character(unit_str),
		as.character(calendar),
		PACKAGE="calencalcs")
	if( retval$error != 0 ) 
		stop("Error in utInvCalendar")

	return(retval$values)
}


