myname = 'remake_SWAssess_fig6.10_c.R'

postscript( paste0(myname,'.ps'), height=7.5, paper='letter' )
par(mfcol=c(3,3))

basedir = 'data'

# orange, green
col_light = c( rgb(231, 156, 124, maxColorValue=255), rgb( 102, 171, 108, maxColorValue=255))
col_dark  = c( rgb(230, 118,  49, maxColorValue=255), rgb(  36, 102,  57, maxColorValue=255))

source( 'dplib_2019-07-29.R' )
library('ncdf4')
library('calencalcs')

globalframenumber <<- 0

#======================================================================================
get_tser = function( fname ) {

	print(paste("Reading", fname ))
	ncid = nc_open( fname )
	var  = nc_main_var( ncid )

	dat = ncvar_get( ncid, var )

	tdim = ncid$dim[[ ncid$unlimdimid ]]
	dates = utCalendar( tdim$vals, tdim$units )

	nc_close( ncid )

	return( list(
		dat	= dat,
		dates	= dates ))

}

#======================================================================================
get_mod_list = function( expt ) {

	fname = paste0( basedir, '/colo/models_sresa2.txt' )
	t = reads( fname, is_char=TRUE )
	return( t )
}

#======================================================================================
do_runoff = function( reg, expts, reg_long ) {

	if( reg == 'cali' )
		ylim = c(0, 12 )
	else if( reg == 'colo' ) 
		ylim = c(0, 4.0 )
	else if( reg == 'riog' ) 
		ylim = c(0, 1.9 )
	else
		stop(paste('ouioiu', reg ))

	varname = 'runoff'

	nexpts = length( expts )

	nxmods = 16
	nxyrs  = 2099 - 1950 + 1

	runoff = array(0., dim=c(nxmods, nexpts, nxyrs))

	for( iexpt in 1:nexpts ) {

		models = get_mod_list( expts[iexpt] )
		nmods = length( models )

		for( imod in 1:nmods ) {
			fname = paste0( basedir, '/', reg, '/', expts[iexpt], '.', models[imod], '/', varname, '/', 
				expts[iexpt], '.', models[imod], '.monthly.', varname, '.avg.nc' )
			if( filesize( fname ) < 10 ) stop(paste("Error, misisng file:", fname))

			rv = get_tser( fname )
			year0 = min( rv$dates$year )
			year1 = max( rv$dates$year )
			nyears = year1 - year0 + 1

			for( iyear in 1:nyears ) {
				year = year0 + iyear - 1
				keep = ((rv$dates$year == year) & (rv$dates$month >= 4) & (rv$dates$month <= 7))
				nkeep = sum( keep )
				if( nkeep != 4 ) stop(paste("BBB error, nkeep should be 2 but is:", nkeep ))
				t2 = rv$dat[ keep ]

				runoff[imod, iexpt, iyear] = sum( t2 )
				}
			}
		}

	#--------------------------
	# Convert from mm to inches
	#--------------------------
	runoff = runoff / 25.4			# inches/day

	dat2plot = runoff
	vartag = 'Apr - Jul runoff'
	dotheplot( reg, expts, dat2plot, ylim, vartag, year0, year1, reg_long ) 
}

#======================================================================================
do_soilmoist = function( reg, expts, reg_long ) {

	if( reg == 'cali' )
		ylim = c(6, 16)
	else if( reg == 'colo' ) 
		ylim = c(8,17)
	else if( reg == 'riog' ) 
		ylim = c(6, 14 )
	else
		stop(paste('oiuzcoiuoi', reg ))

	varname = 'smc'

	nexpts = length( expts )

	nxmods = 16
	nxyrs  = 2099 - 1950 + 1

	smc = array(0., dim=c(nxmods, nexpts, nxyrs))

	for( iexpt in 1:nexpts ) {

		models = get_mod_list( expts[iexpt] )
		nmods = length( models )

		for( imod in 1:nmods ) {
			fname = paste0( basedir, '/', reg, '/', expts[iexpt], '.', models[imod], '/', varname, '/', 
				expts[iexpt], '.', models[imod], '.monthly.', varname, '.avg.nc' )
			if( filesize( fname ) < 10 ) stop(paste("Error, misisng file:", fname))

			rv = get_tser( fname )
			year0 = min( rv$dates$year )
			year1 = max( rv$dates$year )
			nyears = year1 - year0 + 1

			for( iyear in 1:nyears ) {
				year = year0 + iyear - 1
				#-----------------------------------------------------------
				# In the SWE files it says:
				# snow water equivalent in snow pack, state 1st day of month
				# So I'm assuming the Apr value has Apr 1st
				#-----------------------------------------------------------
				#keep = ((rv$dates$year == year) & ( (rv$dates$month == 3) | (rv$dates$month == 4)))
				#nkeep = sum( keep )
				#if( nkeep != 2 ) stop(paste("AAA error, nkeep should be 2 but is:", nkeep ))

				keep = ((rv$dates$year == year) & (rv$dates$month == 6))
				nkeep = sum( keep )
				if( nkeep != 1 ) stop(paste("AAA error, nkeep should be 2 but is:", nkeep ))

				t2 = rv$dat[ keep ]

				smc[imod, iexpt, iyear] = mean( t2 )
				}
			}
		}

	#---------------------
	# Convert mm to inches
	#---------------------
	dat2plot = smc / 25.4	# [nmods, nexpts, nyears]

	print('----')
	print(reg)

	dotheplot( reg, expts, dat2plot, ylim, 'Jun 1 soil moist', year0, year1, reg_long ) 
}


#======================================================================================
do_swe = function( reg, expts, reg_long ) {

	if( reg == 'cali' )
		ylim = c(0, 6.5 )
	else if( reg == 'colo' ) 
		ylim = c(0, 4.0 )
	else if( reg == 'riog' ) 
		ylim = c(0, 1.2 )
	else
		stop(paste('oiuzcoiuoi', reg ))

	varname = 'swe'

	nexpts = length( expts )

	nxmods = 16
	nxyrs  = 2099 - 1950 + 1

	swe_apr1 = array(0., dim=c(nxmods, nexpts, nxyrs))

	for( iexpt in 1:nexpts ) {

		models = get_mod_list( expts[iexpt] )
		nmods = length( models )

		for( imod in 1:nmods ) {
			fname = paste0( basedir, '/', reg, '/', expts[iexpt], '.', models[imod], '/', varname, '/', 
				expts[iexpt], '.', models[imod], '.monthly.', varname, '.avg.nc' )
			if( filesize( fname ) < 10 ) stop(paste("Error, misisng file:", fname))

			rv = get_tser( fname )
			year0 = min( rv$dates$year )
			year1 = max( rv$dates$year )
			nyears = year1 - year0 + 1

			for( iyear in 1:nyears ) {
				year = year0 + iyear - 1
				#-----------------------------------------------------------
				# In the SWE files it says:
				# snow water equivalent in snow pack, state 1st day of month
				# So I'm assuming the Apr value has Apr 1st
				#-----------------------------------------------------------
				keep = ((rv$dates$year == year) & (rv$dates$month == 4))
				nkeep = sum( keep )
				if( nkeep != 1 ) stop(paste("AAA error, nkeep should be 1 but is:", nkeep ))

				t2 = rv$dat[ keep ]

				swe_apr1[imod, iexpt, iyear] = mean( t2 )
				}
			}
		}

	#---------------------
	# Convert mm to inches
	#---------------------
	dat2plot = swe_apr1 / 25.4	# [nmods, nexpts, nyears]

	dotheplot( reg, expts, dat2plot, ylim, 'Apr 1 SWE', year0, year1, reg_long ) 
}

#=======================================================================================================
# dat2plot[nmods,nexpts,nyears]
#
dotheplot = function( reg, expts, dat2plot, ylim, vartag, year0, year1, reg_long ) {

	nexpts = length( expts )

	nmods  = dim( dat2plot )[1]
	nyears = dim( dat2plot )[3]

	canvas( xlim=c(1950, 2100), ylim=ylim,
		xaxs='i', yaxs='i', 
		xlab='year', ylab='Inches',
		grid=FALSE,
		axes=TRUE,
		box=FALSE,
		main=paste(reg_long, vartag ))
	abline(h=ylim[1], lwd=2)
	abline(v=1950, lwd=2)

	tvals = year0:year1

	y1hist = 2005
	ixhist = wherenearest( tvals, y1hist )

	tvals_hist = 1950:y1hist
	tvals_fut = (y1hist+1):2099

	for( iexpt in 1:nexpts )
	for( imod in 1:nmods ) {
		yy = dat2plot[imod,iexpt,]

		tser_hist = yy[1:ixhist]
		tser_fut  = yy[(ixhist+1):nyears]

		lwd = 1
		if( iexpt == 1 ) 
			lines( tvals_hist, tser_hist, col=grey(0.6), lwd=lwd )

		lines( tvals_fut, tser_fut, col=col_light[iexpt], lwd=lwd )
		}

	#--------------
	# Add hist line
	#--------------
	keep = (tvals <= y1hist)
	hmean = mean( dat2plot[,1,keep] )
	abline( h = hmean )

	#----------
	# Hist MMEA
	#----------
	nthist = sum( keep )
	yy = array(0, nthist)
	for( it in 1:nthist ) {
		yy[it] = mean( dat2plot[,1,it] )
		}
	lines( tvals_hist, yy, lwd=2 )

	#---------
	# Fut MMEA
	#---------
	keep = (tvals > y1hist )
	for( iexpt in 1:nexpts ) {
		ntfut = sum( keep )
		yy = array(0, ntfut )
		for( it in 1:ntfut ) {
			yy[it] = mean( dat2plot[,iexpt,ixhist+it] )
			}
		lines( tvals_fut, yy, lwd=2, col=col_dark[iexpt] )
		}
}

#======================================================================================
procreg = function( reg, expts, reg_long ) {

	do_swe		( reg, expts, reg_long )
	do_runoff	( reg, expts, reg_long )
	do_soilmoist	( reg, expts, reg_long )
}

#======================================================================================
calc_dat = function( opts ) {

	dat = list()

	reg = c( 'cali', 'colo', 'riog' )
	reg_long = c( 'California', 'Colorado', 'Rio Grande' )
	nreg = length( reg )

	expts = c( 'sresa2', 'sresb1' )

	for( ireg in 1:nreg ) {
		procreg( reg[ireg], expts, reg_long[ireg] )
		}

	return(dat)
}

#======================================================================================
plotup = function( dat, opts, po ) {

}

#======================================================================================
doit = function() {

	tdate = date()
	print(paste("==================", myname, "starting at", tdate, '==============' ))

	use_cache = FALSE
	cache_fname = paste(myname,'_Rcache', sep='' )
	cache_file_exists = (filesize(cache_fname) > 10)

	#-----------------
	# Set data options
	#-----------------
	opts = list()

	#--------------------
	# Use cache, if asked
	#--------------------
	if( use_cache ) {
		if( cache_file_exists ) {
			print(paste("loading data from cache file", cache_fname))
			load( file=cache_fname )
			print('...done')
			do_calc_dat = FALSE
			}
		else
			do_calc_dat = TRUE
		}
	else
		do_calc_dat = TRUE

	#-------------------
	# Calculate the data
	#-------------------
	if( do_calc_dat )
		dat = calc_dat( opts )
		
	#-----------------------------------------------------
	# Save to cache if requested and doesn't exist already
	#-----------------------------------------------------
	if( use_cache && (! cache_file_exists))
		save( file=cache_fname, dat, opts )
			
	#---------------------------------
	# Set plot options and plot result
	#---------------------------------
	po = list()
	plotup( dat, opts, po )
}

#======================================================================================

doit()
dev.off()
